;===========================================================
; Atari 5200 "Jum's Pong" code
; Written by James Higgs, February 2000
; email: jum@emulationworld.com
;
; Initialisation code and inspiration from Dan Boris'
; PM example. Thanks Dan!
;
; Assemble with DASM
;
; PM base set to:	2000
; DLIST addr:		1000
; DLIST data addr:	1100
; charset data addr:	1800
;
; TO DO:
; 1. Fix start key handling (latch?)
; 2. Sound :)
;
; WARNING: Amateur 6502 assembly code. Optimise and clean
; at your own discretion.
;
; Most code copyright James Higgs 2000.
; Binary and source may only be distributed FREE of charge!
; 
;===========================================================

        processor 6502

; ************ Hardware Adresses ***************************

DMACTL  equ     $D400           ;DMA Control
sDMACTL equ     $07             ;DMA Control Shadow
PMBASE  equ     $D407           ;PM base address
CHBASE  equ     $D409           ;Character set base
GRACTL  equ     $C01D           ;Graphics control
PRIOR   equ     $C01B           ;PM priorities
CONSOL	equ	$C01F
SIZEP0  equ     $C008           ;Size of player 0
SIZEP1	equ	$C009		;Size of player 1
HPOSP0  equ     $C000           ;Horizontal position player 0
HPOSP1	equ	$C001		;horiz posn player 1
HPOSM0	equ	$C004
HPOSM1	equ	$C005
HPOSM2	equ	$C006
HPOSM3	equ	$C007
SIZEM	equ	$C00C
M2PF	equ	$C002		; read
M3PF	equ	$C003		; read
M2PL	equ	$C00A		; read
M3PL	equ	$C00B		; read
HITCLR	equ	$C01E		; write
TRIG0	equ	$C010		; read
TRIG1	equ	$C011		; read
DLISTL  equ     $D402           ;Display list lo
DLISTH  equ     $D403           ;Display list hi
sDLISTL equ     $05             ;Display list lo shadow
sDLISTH equ     $06             ;Display list hi shadow
CHACTL  equ     $D401           ;Character control
NMIEN   equ     $D40E           ;NMI Enable
KBCODE	equ	$E809
ATIMER	equ	$04		; attract mode timer	
sCOLPM0 equ      $08            ;Player/missile 0 color shadow
sCOLPM1 equ      $09            ;Player/missile 0 color shadow
sCOLPM2 equ      $0A            ;Player/missile 0 color shadow
sCOLPM3 equ      $0B            ;Player/missile 0 color shadow
sCOLOR0  equ     $0C             ;Color 0 shadow
sCOLOR1  equ     $0D             ;Color 1 shadow
sCOLOR2  equ     $0E             ;Color 2 shadow
sCOLOR3  equ     $0F             ;Color 3 shadow
sCOLBK	equ	$10		;COLBK shadow
sPOT0	equ	$11
sPOT1	equ	$12
sPOT2	equ	$13
sPOT3	equ	$14

; variables
p0y	equ	$C0		; player 0 y-coord
p1y	equ	$C1		; player 1 y-coord
ballx	equ	$C2		; ball x-coord
bally	equ	$C3		; ball y-coord
balldx	equ	$C4
balldy	equ	$C5
p0sc0	equ	$C8
p0sc1	equ	$C9
p1sc0	equ	$CA
p1sc1	equ	$CB
top	equ	$CC		; upper and lower limit
bottom	equ	$CD
server	equ	$CE		; next to serve ball
tmp1	equ	$E0		; temp vars
tmp2	equ	$E1
tmp3	equ	$E2
prntlo	equ	$FE		; print dest lo addr
prnthi	equ	$FF		; print dest hi addr	
prntcol	equ	$FD		; print colour (bit 6 and 7)
textlo	equ	$FA		; text src addr lo
texthi	equ	$FB		; text src addr hi
prntlen	equ	$FC		; length to print

;*************** Start of Code *******************

        org     $4000           ;Start of cartridge area
Start
        sei                     ;Disable interrupts
        cld                     ;Clear decimal mode

;************** Clear zero page and hardware ******

        lda     #$00
        tax
crloop1    
        sta     $00,x           ;Clear zero page
        sta     $D400,x         ;Clear ANTIC
        sta     $C000,x         ;Clear GTIA
        sta     $E800,x         ;Clear POKEY
        dex
        bne     crloop1

;************* Clear RAM **************************

        ldy     #$00            ;Clear Ram
        lda     #$02            ;Start at $0200
        sta     $81             
        lda     #$00
        sta     $80
crloop3
        lda     #$00
        sta     ($80),y         ;Store data
        iny                     ;Next byte
        bne     crloop3         ;Branch if not done page
        inc     $81             ;Next page
        lda     $81
        cmp     #$40            ;Check if end of RAM
        bne     crloop3         ;Branch if not

;************* Setup display list *******************

        ldx     #$21            ;Number of bytes in list
dlloop                          ;Copy display list to RAM
        lda     dlist,x         ;Get byte
        sta     $1000,x         ;Copy to RAM
        dex                     ;next byte
        bpl     dlloop

;************ Setup IRQ vectors *********************

        lda     #$03            ;point IRQ vector
        sta     $200            ;to BIOS routine
        lda     #$FC
        sta     $201
        lda     #$B8            ;point VBI vector
        sta     $202            ;to BIOS routine
        lda     #$FC
        sta     $203
        lda     #$B2            ;point Deferred VBI
        sta     $204            ;to BIOS routine
        lda     #$FC
        sta     $205

;************* Setup hardware registers *************

        lda     #$22            ;Set color PF0
        sta     sCOLOR0
        lda     #$0F            ;Set color PF1
        sta     sCOLOR1
        lda     #$84            ;Set color PF2
        sta     sCOLOR2             
        lda     #$00            ;Set Display list pointer
        sta     sDLISTL
        sta     DLISTL
        lda     #$10
        sta     sDLISTH
        sta     DLISTH
        lda     #$ba            ;Set Charcter Set Base
        sta     CHBASE
        lda     #$22            ;Enable DMA
        sta     sDMACTL
        lda     #$40            ;Enable NMI
        sta     NMIEN


;************* Setup Player/Missile registers ***************

        lda     #$3E           ;Enable DMA (single line resolution/
        sta     sDMACTL        ;normal background)
        lda     #$20           ;Set PM base address ($2000)
        sta     PMBASE
        lda     #$03           ;Enable players and missiles
        sta     GRACTL
        lda     #$2E           ;Color of player 0
        sta     sCOLPM0
        lda     #$5E           ;Color of player 1
        sta     sCOLPM1
        lda     #$0E           ;Color of player 2
        sta     sCOLPM2
        lda     #$0A           ;Color of player 3
        sta     sCOLPM3
        ldy     #$00
        lda     #$00           ;Size of player 0
        sta     SIZEP0
        sta     SIZEP1
	sta	SIZEM
	lda	#$30
	sta	HPOSP0		; $30 = left edge
	lda	#$C8
	sta	HPOSP1		; $CE = right edge
	lda	#$80
	sta	HPOSM3
	lda	#$82
	sta	HPOSM2
        lda     #$04           ;Give playfield priority over players
        sta     PRIOR


;************ Draw Playfield **************************************

	lda	#$00		; text dest. addr
	sta	prntlo
	lda	#$18
	sta	prnthi
	lda	#$40		; colour of text
	sta	prntcol
	lda	#$00		; text src addr
	sta	textlo
	lda	#$b8
	sta	texthi
	lda	#40		; text length
	sta	prntlen
	jsr	print		; print title and horizontal bar
	lda	#$90
	sta	prntlo
	lda	#$19
	sta	prnthi
	lda	#$14
	sta	textlo
	lda	#20
	sta	prntlen
	jsr	print		; print bottom horiz bar	
	

;*******************************************************************
;                            MAIN LOOP
;*******************************************************************
main
	jsr	title
	jsr	play
	jsr	gameover
	jmp	main

;************ Display Title / Wait for start ***********************
title
	; set background colour
	lda	#$96
	sta	sCOLBK

	; 1. display PRESS START
	lda	#$04		; text dest. addr
	sta	prntlo
	lda	#$19
	sta	prnthi
	lda	#$40		; colour of text
	sta	prntcol
	lda	#$28		; text src addr
	sta	textlo
	lda	#20		; text length
	sta	prntlen
	jsr	print		; print PRESS START

	; 2. check for start button
	; set CONSOL bits 0 and 1 to 0 (select contr 0)
	lda	CONSOL
	and	#$FC
	sta	CONSOL
waitstart
	jsr waitvb
	lda	KBCODE
	and	#$1E
	;cmp	#$09
	cmp	#$18
	bne	waitstart

	; 3. clear START display
	lda	#$3C		; text src addr
	sta	textlo
	jsr	print		; print 20 spaces
	
	; 4. reset scores
	lda	#0
	sta	p0sc0
	sta	p0sc1
	sta	p1sc0
	sta	p1sc1
	
	; set server to player 0
	sta	server

	; reset collision regs
	sta	HITCLR

	rts

;************ Run Game *********************************************
play
	; set background colour
	lda	#$00
	sta	sCOLBK

	lda	#$30		; set p0 vert posn
	sta	p0y
	lda	#$80
	sta	p1y
	lda	#50
	sta	top
	lda	#220
	sta	bottom

newball
	jsr	waitvb
	jsr	drawscore
	; check if game is over
	lda	p0sc0
	cmp	#$02
	bne	checkover2
	rts			; return to main if p0 score = 20
checkover2
	lda	p1sc0
	cmp	#$02
	bne	startserve
	rts			; return to main if p1 score = 20

startserve
	; check which player gets to serve
	lda	server
	bne	pl1serve
pl0serve
	; wait for trig pl0 = 0
	jsr	waitvb
	jsr	drawpl0
	jsr	drawpl1
	lda	TRIG0
	bne	pl0serve
	lda	#$01		; set up ball dx and dy
	sta	balldx
	jmp	doserve

pl1serve
	; wait for trig pl1 = 0
	jsr	waitvb
	jsr	drawpl0
	jsr	drawpl1
	lda	TRIG1
	bne	pl1serve
	lda	#$FF		; ball serve pl 1 dx
	sta	balldx

doserve
	lda	#$00		; ball serve dy
	sta	balldy
	lda	#148
	sta	bally
	sta	ballx

	ldx	#180
	
        ldx     #$20            ;Starting position of player
mvloop1
        jsr     waitvb          ;Wait for a vertical bank
	jsr	drawpl0
	jsr	drawpl1
	jsr	drawmiss
	jsr	drawscore

	; check for collision with PF or PL's
checkpfcoll
	lda	M2PF
	beq	checkplcoll0	; no PF coll

pfcoll
	sta	HITCLR		; clear coll regs
	; bounce ball vertically
	lda	bally
	bmi	bouncebottom

bouncetop
	; top bounce code here
	; NB: needs to be fixed!
	lda	#$01
	;sec
	;sbc	balldy
	sta	balldy
	;rts				; debug
	;inc	balldy
	jmp	checkplcoll0

bouncebottom
	; bottom bounce code here
	; NB: needs to be fixed!
	lda	#$FF
	sec
	;sbc	balldy
	sta	balldy
	;inc	balldy

checkplcoll0				; collision with PL0?
	lda	M3PL
	beq	checkplcoll1		; no PL0 coll
        ; player0 to ball collision code
	; ie: bounce ball off pl0 bat at the right angle
        lda     bally                   ; A = (bally-baty) / 4
	clc
        sbc     sPOT1
        and     #$1F
	clc
        ror
	ror
        sta     tmp1			; save A (check location!)
        lda     balldy
        sec
        adc     #$02                    ; A = (balldy + 2) * 8
        clc     
        rol
        rol
        rol
        adc     tmp1			; add saved var
        tax                             ; x = table index
        lda     table1,x		; get new ball dy from table
        sta     balldy
	lda	#$01			; ball dx = 1
	sta	balldx
	jmp	checkedge
	
checkplcoll1				; collision with PL1?
	lda	M2PL
	beq	checkedge			; no PL1 coll
        ; player1 to ball collision code
	; ie: bounce ball off pl1 bat at the right angle
        lda     bally                   ; A = (bally-baty) / 4
	clc
        sbc     sPOT3
        and     #$1F
	clc
        ror
        ror
        sta     tmp1			; save A (check location!)
        lda     balldy
        sec
        adc     #$02                    ; A = (balldy + 2) * 8
        clc     
        rol
        rol
        rol
	sta	tmp2
        adc     tmp1			; add saved var
; debug
	sta	tmp3
; end debug
        tax                             ; x = table index
        lda     table1,x		; get new ball dy from table
        sta     balldy
	lda	#$FF			; ball dx = 1
	sta	balldx

checkedge
	lda	ballx
	and	#$E0		; get high 3 bits
	bne	checkright
	jsr	sc1inc		; zone = 0
	lda	#1
	sta	server
	jmp	newball

checkright
	cmp	#$E0		; is ballx in 224-255 zone
	bne	mvball
	jsr	sc0inc
	lda	#0
	sta	server
	jmp	newball		
	
mvball
	lda	ballx		; move ball horiz
	clc
	adc	balldx
	sta	ballx
	sta	HPOSM3
	adc	#2
	sta	HPOSM2
	lda	bally		; move ball vert
	clc
	adc	balldy
	sta	bally
	
	sta	HITCLR		; clear collision regs
	jmp	mvloop1
	

; *********** Draw Player 0 **********************************
drawpl0
	txa			; save x and y
	pha
	tya
	pha

	;lda	p0y		; get p0 y-pos
	ldx	sPOT1
	lda	#$00
        ldy     #$00
d0loop1         
        sta     $2400,y         ; Write 0 to player 0 RAM
        iny
	dex
        bne     d0loop1

	lda	#$3F		; Paddle gfx data
	ldx	#$20		; Height of paddle
d0loop2
        sta     $2400,y         ; Write #$FF to player 0 RAM
        iny
	dex
        bne     d0loop2

	lda	#$00
d0loop3
        sta     $2400,y         ; Write 0 to player 0 RAM
        iny
        bne     d0loop3		; until y = 256

	pla			; restore x an dy
	tay
	pla
	tax

	rts

; *********** Draw Player 1 **********************************
drawpl1
	txa			; save x and y
	pha
	tya
	pha

	ldx	sPOT3		; get p1 y-pos
	lda	#$00
        ldy     #$00
d1loop1         
        sta     $2500,y         ; Write 0 to player 1 RAM
        iny
	dex
        bne     d1loop1

	lda	#$FC		; Paddle gfx data
	ldx	#$20		; Height of paddle
d1loop2
        sta     $2500,y         ; Write #$FF to player 1 RAM
        iny
	dex
        bne     d1loop2

	lda	#$00
d1loop3
        sta     $2500,y         ; Write 0 to player 1 RAM
        iny
        bne     d1loop3		; until y = 256

	pla			; restore x an dy
	tay
	pla
	tax

	rts

; *********** Draw Missiles **********************************
drawmiss
	txa			; save x and y
	pha
	tya
	pha

	ldx	bally		; get ball y-pos
	lda	#$00
        ldy     #$00
dmloop1         
        sta     $2300,y         ; Write 0 to missile RAM
        iny
	dex
        bne     dmloop1

	ldx	#$04		; Height of ball
dmloop2
	lda	ball1,x
        sta     $2300,y         ; Write ball data to missile RAM
        iny
	dex
        bne     dmloop2

	lda	#$00
dmloop3
        sta     $2300,y         ; Write 0 to missile RAM
        iny
        bne     dmloop3		; until y = 256

	pla			; restore x an dy
	tay
	pla
	tax

	rts

; *********** inc score pl 0 *********************************
sc0inc
	lda	#10
	inc	p0sc1		; inc lo digit
	cmp	p0sc1		; check if digit is 10
	bne	end1
	inc	p0sc0		; inc hi digit
	lda	#0		; reset lo digit
	sta	p0sc1	
end1
	rts

; *********** inc score pl 1 *********************************
sc1inc
	lda	#10
	inc	p1sc1		; inc lo digit
	cmp	p1sc1		; check if digit is 10
	bne	end2
	inc	p1sc0		; inc hi digit
	lda	#0		; reset lo digit
	sta	p1sc1	
end2
	rts

; *********** Draw Scores ************************************
; ND: add colour by or'ing with two top bits
drawscore
	clc
	lda	p0sc0		; pl0 hi digit
	adc	#16
	sta	$1800
	lda	p0sc1		; pl0 lo digit
	adc	#16
	sta	$1801
	lda	p1sc0		; pl1 hi digit
	adc	#16
	sta	$1812
	lda	p1sc1		; pl1 lo digit
	adc	#16
	sta	$1813

	rts	

;************ Display Game Over **************************
gameover
	; set background colour
	lda	#$b6
	sta	sCOLBK

	; 1. display PRESS START
	lda	#$04		; text dest. addr
	sta	prntlo
	lda	#$19
	sta	prnthi
	lda	#$40		; colour of text
	sta	prntcol
	lda	#$50		; text src addr
	sta	textlo
	lda	#20		; text length
	sta	prntlen
	jsr	print		; print GAME OVER

	ldx	#200		; wait for 4 seconds
waitloop
	jsr	waitvb
	dex
	bne	waitloop

	rts

; *********** Print a string *********************************
; colour in bits 6 and 7 of prtcol ($FD)

print
        ldy     #$00           
        cld
prloop
        lda     (textlo),y         ;Get character
        sec
        sbc     #$20            ;Convert to ATASCII
	ora	prntcol		; set colour from bits 6 & 7
        sta     (prntlo),y	;Store in video memory
        iny                     ;Next character
        cpy     prntlen		             
        bne     prloop

	rts

;************ Wait for vertical blank ************************

waitvb
        lda     $02     ;Read timer (this is incremented during VB)
waitvb2
        cmp     $02         ;Did it change?
        beq     waitvb2     ;If not keep waiting
        rts

;************* Display list data ****************************

        org     $b000
dlist   .byte     $70,$70,$70      ;24 blank scanlines
        .byte     $46,$00,$18      ;Mode 6 and Load memory scan $1800
        .byte     $06,$06,$06,$06,$06,$06,$06  ;23 more line of mode 6
        .byte     $06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06,$06
        .byte     $06,$06,$06
        .byte     $41,$00,$10       ;Jump back to start at $1000

;************* Ball shape (missiles 0 and 1) ****************
; maybe put a highlight on the ball using missile 2 or 3?

	org	$b7F8
ball1	.byte	%00000000		; dummy line
	.byte	%01100000		; data for missiles
        .byte	%11110000		; 4 and 3
        .byte	%11110000
        .byte	%01100000


;************* Text data ************************************
         org     $b800
text1   .byte	"00   JUM'S PONG   00"
	.byte	"____________________"

text2	.byte	"    PRESS  START    "

text3	.byte	"                    "

text4	.byte	"     GAME  OVER     "


;************* Ball-bat bounce table ************************
        ; Look-up table for calculating departing dy of ball
        ; from bat.
        org     $b900
table1  .byte   $FE,$FE,$FE,$FE,$FE,$FE,$FF,$00         ; -2
        .byte   $FE,$FE,$FF,$FF,$FF,$FF,$00,$01         ; -1
        .byte   $FE,$FF,$00,$00,$00,$00,$01,$02         ;  0
        .byte   $FF,$00,$01,$01,$01,$01,$02,$02         ;  1
        .byte   $00,$01,$02,$02,$02,$02,$02,$02         ;  2


; My Pong Charset
	org	$ba00
	include	"pongchr2.asm"

;************** Cart reset vector **************************

        org     $bffd
        .byte   $FF         ;Don't display Atari logo
        .byte   $00,$40     ;Start code at $4000


; Paddle moving code
; (double up for pad 0 and 1)
;
;        lda     sPAD0
;        sla     #$04                    ; / 16
;        clc
;        sbc     #$08
;        bcs             
